"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _isNull2 = _interopRequireDefault(require("lodash/isNull"));

var _isBoolean2 = _interopRequireDefault(require("lodash/isBoolean"));

var _isNumber2 = _interopRequireDefault(require("lodash/isNumber"));

var _isString2 = _interopRequireDefault(require("lodash/isString"));

var _isEmpty2 = _interopRequireDefault(require("lodash/isEmpty"));

var _attribute = _interopRequireDefault(require("./../attribute"));

var _modelError = _interopRequireDefault(require("./../modelError"));

class ArrayAttribute extends _attribute.default {
  /**
   * If value is assigned (checked in the parent validate call), it must by an instance of array.
   */
  async validateType(value) {
    if (Array.isArray(value)) {
      return;
    }

    this.expected("array", typeof value);
  }
  /**
   * @returns {Promise<void>}
   */


  async validateValue(value) {
    const currentValue = value;
    const errors = [];

    for (let i = 0; i < currentValue.length; i++) {
      const current = currentValue[i];

      if (!this.__isPrimitiveValue(current)) {
        errors.push({
          code: _modelError.default.INVALID_ATTRIBUTE,
          data: {
            index: i
          },
          message: `Validation failed, item at index ${i} not a primitive value.`
        });
      }
    }

    if (!(0, _isEmpty2.default)(errors)) {
      throw new _modelError.default("Validation failed.", _modelError.default.INVALID_ATTRIBUTE, errors);
    }
  }

  __isPrimitiveValue(value) {
    return (0, _isString2.default)(value) || (0, _isNumber2.default)(value) || (0, _isBoolean2.default)(value) || (0, _isNull2.default)(value);
  }

}

var _default = ArrayAttribute;
exports.default = _default;
//# sourceMappingURL=arrayAttribute.js.map