"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _isObject2 = _interopRequireDefault(require("lodash/isObject"));

var _attribute = _interopRequireDefault(require("./../attribute"));

var _modelAttributeValue = _interopRequireDefault(require("./modelAttributeValue"));

var _model = _interopRequireDefault(require("./../model"));

var _index = require("../index");

class ModelAttribute extends _attribute.default {
  constructor(name, attributesContainer, model) {
    super(name, attributesContainer); // If provided class is not a subclass of Model, we must throw an error.

    const validInstance = model.prototype instanceof _model.default;

    if (!validInstance) {
      throw Error(`"model" attribute "${name}" received an invalid class (subclass of Model is required).`);
    }

    this.modelClass = model;
  }

  getModelClass() {
    return this.modelClass;
  }

  getModelInstance() {
    const modelClass = this.getModelClass();
    return new modelClass();
  }

  setValue(value) {
    if (!this.canSetValue()) {
      return;
    }

    const finalValue = this.onSetCallback(value);
    let newValue = null;

    if (finalValue instanceof _model.default) {
      newValue = finalValue;
    } else if ((0, _isObject2.default)(finalValue)) {
      newValue = this.getModelInstance();
      newValue.populate(finalValue);
    } else {
      newValue = finalValue;
    }

    this.value.setCurrent(newValue);
  }

  async getJSONValue() {
    const value = this.getValue();

    if (value instanceof _model.default) {
      return {};
    }

    return value;
  }

  async getStorageValue() {
    const value = await this.getValue();

    if (value instanceof _model.default) {
      const json = {};

      for (let name in value.getAttributes()) {
        const attribute = value.getAttribute(name); // $FlowFixMe - we can be sure we have attribute because it's pulled from list of attributes, using getAttributes() method.

        if (attribute.getToStorage() && !attribute.getDynamic()) {
          // $FlowFixMe - we can be sure we have attribute because it's pulled from list of attributes, using getAttributes() method.
          json[name] = await attribute.getStorageValue();
        }
      }

      return json;
    }

    return value;
  }

  setStorageValue(value) {
    const newValue = this.getModelInstance().populateFromStorage(value);
    this.value.setCurrent(newValue, {
      skipDifferenceCheck: true
    });
    return this;
  }
  /**
   * If value is assigned (checked in the parent validate call), it must by an instance of Model.
   */


  async validateType(value) {
    if (_model.default.isInstanceOf(value, this.getModelClass())) {
      return;
    }

    this.expected("instance of Model class", typeof value);
  }
  /**
   * We can be sure that value is an instance of Model since otherwise this point would not be reached
   * (because of the validateType method above).
   * @param value
   * @returns {Promise<void>}
   */


  async validateValue(value) {
    const currentValue = value;
    await currentValue.validate();
  }
  /**
   * Returns AttributeValue class to be used on construct.
   * @returns {AttributeValue}
   */


  getAttributeValueClass() {
    return _modelAttributeValue.default;
  }

}

var _default = ModelAttribute;
exports.default = _default;
//# sourceMappingURL=modelAttribute.js.map