"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _isEmpty2 = _interopRequireDefault(require("lodash/isEmpty"));

var _clone2 = _interopRequireDefault(require("lodash/clone"));

var _isPlainObject2 = _interopRequireDefault(require("lodash/isPlainObject"));

var _attribute = _interopRequireDefault(require("./../attribute"));

var _model = _interopRequireDefault(require("./../model"));

var _modelError = _interopRequireDefault(require("./../modelError"));

var _index = require("../index");

class ModelsAttribute extends _attribute.default {
  constructor(name, attributesContainer, model, keyValue = false) {
    super(name, attributesContainer); // If provided class is not a subclass of Model, we must throw an error.

    const validInstance = model.prototype instanceof _model.default;

    if (!validInstance) {
      throw Error(`"models" attribute "${name}" received an invalid class (subclass of Model is required).`);
    }

    this.modelClass = model;
    this.keyValue = keyValue;
    this.value.current = this.keyValue ? {} : [];
  }

  getModelClass() {
    return this.modelClass;
  }

  getModelInstance() {
    const modelClass = this.getModelClass();
    return new modelClass();
  }

  setValue(values) {
    if (!this.canSetValue()) {
      return;
    }

    const finalValues = this.onSetCallback(values);

    if (this.keyValue) {
      // Even if the value is invalid (eg. a string), we allow it here, but calling validate() will fail.
      if (!(0, _isPlainObject2.default)(finalValues)) {
        this.value.setCurrent(finalValues);
        return;
      }

      let newValues = {};

      for (let key in finalValues) {
        if ((0, _isPlainObject2.default)(finalValues[key])) {
          const newValue = this.getModelInstance();
          newValue.populate((0, _clone2.default)(finalValues[key]));
          newValues[key] = newValue;
        } else {
          newValues[key] = finalValues[key];
        }
      }

      this.value.setCurrent(newValues);
    } else {
      // Even if the value is invalid (eg. a string), we allow it here, but calling validate() will fail.
      if (!(finalValues instanceof Array)) {
        this.value.setCurrent(finalValues);
        return;
      }

      let newValues = [];

      for (let i = 0; i < finalValues.length; i++) {
        if ((0, _isPlainObject2.default)(finalValues[i])) {
          const newValue = this.getModelInstance();
          newValue.populate((0, _clone2.default)(finalValues[i]));
          newValues.push(newValue);
        } else {
          newValues.push(finalValues[i]);
        }
      }

      this.value.setCurrent(newValues);
    }
  }

  async getJSONValue() {
    const value = this.value.getCurrent();

    if (this.keyValue) {
      if (value instanceof Object) {
        return {};
      }

      return value;
    } else {
      if (value instanceof Array) {
        const json = [];

        for (let i = 0; i < value.length; i++) {
          json.push({});
        }

        return json;
      }

      return value;
    }
  }

  async getStorageValue() {
    const value = this.getValue();

    if (this.keyValue) {
      if (value instanceof Object) {
        const data = {};

        for (let key in value) {
          const model = value[key];
          const json = {};

          for (let name in model.getAttributes()) {
            const attribute = model.getAttribute(name); // $FlowFixMe - we can be sure we have attribute because it's pulled from list of attributes, using getAttributes() method.

            if (attribute.getToStorage()) {
              // $FlowFixMe - we can be sure we have attribute because it's pulled from list of attributes, using getAttributes() method.
              json[name] = await attribute.getStorageValue();
            }
          }

          data[key] = json;
        }

        return data;
      }

      return {};
    } else {
      if (value instanceof Array) {
        const data = [];

        for (let i = 0; i < value.length; i++) {
          const model = value[i];
          const json = {};

          for (let name in model.getAttributes()) {
            const attribute = model.getAttribute(name); // $FlowFixMe - we can be sure we have attribute because it's pulled from list of attributes, using getAttributes() method.

            if (attribute.getToStorage() && !attribute.getDynamic()) {
              // $FlowFixMe - we can be sure we have attribute because it's pulled from list of attributes, using getAttributes() method.
              json[name] = await attribute.getStorageValue();
            }
          }

          data.push(json);
        }

        return data;
      }

      return [];
    }
  }

  setStorageValue(value) {
    if (this.keyValue) {
      const newValue = {};

      if (value instanceof Object) {
        for (let key in value) {
          const item = value[key];
          newValue[key] = this.getModelInstance().populateFromStorage(item);
        }

        this.value.setCurrent(newValue, {
          skipDifferenceCheck: true
        });
      }
    } else {
      const newValue = [];

      if (Array.isArray(value)) {
        value.forEach(item => {
          newValue.push(this.getModelInstance().populateFromStorage(item));
        });
        this.value.setCurrent(newValue, {
          skipDifferenceCheck: true
        });
      }
    }

    return this;
  }
  /**
   * If value is assigned (checked in the parent validate call), it must by an instance of Model.
   */


  async validateType(value) {
    if (this.keyValue) {
      if (value instanceof Object) {
        return;
      }

      this.expected("object", typeof value);
    } else {
      if (value instanceof Array) {
        return;
      }

      this.expected("array", typeof value);
    }
  }

  async validateValue(value) {
    if (this.keyValue) {
      const objectValue = value; // This validates on all of the model's levels.

      const errors = [];

      for (let key in objectValue) {
        const current = objectValue[key];

        if (!_model.default.isInstanceOf(current, this.getModelClass())) {
          errors.push({
            code: _modelError.default.INVALID_ATTRIBUTE,
            data: {
              index: key
            },
            message: `Validation failed, item at index ${key} not an instance of Model class.`
          });
          continue;
        }

        try {
          await current.validate();
        } catch (e) {
          errors.push({
            code: e.code,
            data: (0, _objectSpread2.default)({
              index: key
            }, e.data),
            message: e.message
          });
        }
      }

      if (!(0, _isEmpty2.default)(errors)) {
        throw new _modelError.default("Validation failed.", _modelError.default.INVALID_ATTRIBUTE, errors);
      }
    } else {
      const arrayValue = value; // This validates on all of the model's levels.

      const errors = [];

      for (let i = 0; i < arrayValue.length; i++) {
        const current = arrayValue[i];

        if (!_model.default.isInstanceOf(current, this.getModelClass())) {
          errors.push({
            code: _modelError.default.INVALID_ATTRIBUTE,
            data: {
              index: i
            },
            message: `Validation failed, item at index ${i} not an instance of Model class.`
          });
          continue;
        }

        try {
          await current.validate();
        } catch (e) {
          errors.push({
            code: e.code,
            data: (0, _objectSpread2.default)({
              index: i
            }, e.data),
            message: e.message
          });
        }
      }

      if (!(0, _isEmpty2.default)(errors)) {
        throw new _modelError.default("Validation failed.", _modelError.default.INVALID_ATTRIBUTE, errors);
      }
    }
  }

}

var _default = ModelsAttribute;
exports.default = _default;
//# sourceMappingURL=modelsAttribute.js.map